#!/usr/bin/env python
# encoding: utf-8
"""
Demonstration of plotting FSPS's un-normalized filter transmission tables
(i.e., contents of $SPS_HOME/data/all_filters.dat).
"""

import fsps

from matplotlib.figure import Figure
from matplotlib.backends.backend_agg import FigureCanvasAgg as FigureCanvas
import matplotlib.gridspec as gridspec


names = ['sdss_u', 'sdss_g', 'sdss_r', 'sdss_i',
         '2mass_J', '2mass_H', '2mass_Ks']
shortnames = ['u', 'g', 'r', 'i', 'J', 'H', 'Ks']
colors = ['violet', 'dodgerblue', 'maroon', 'black', 'c', 'm', 'y']
filters = [fsps.get_filter(n) for n in names]

fig = Figure(figsize=(3.5, 3.5))
canvas = FigureCanvas(fig)
gs = gridspec.GridSpec(
    1, 1, left=0.17, right=0.95, bottom=0.15, top=0.95,
    wspace=None, hspace=None, width_ratios=None, height_ratios=None)
ax = fig.add_subplot(gs[0])

for name, fltr, c, shortname in zip(names, filters, colors, shortnames):
    lmbd, trans = fltr.transmission
    lambda_eff = fltr.lambda_eff / 10000.  # µm
    ax.plot(lmbd / 10000., trans, ls='-', lw=1., c=c)
    ax.annotate(shortname, (lambda_eff, trans.max()),
                textcoords='offset points',
                xytext=(0., 5.),
                ha='center', va='bottom')

ax.set_ylim(0., 1.1)
ax.set_xlabel(r"$\lambda~\mu\mathrm{m}$")
ax.set_ylabel(r"$T$")
gs.tight_layout(fig, pad=1.08, h_pad=None, w_pad=None, rect=None)
canvas.print_figure("sdss_2mass_transmission_demo.pdf", format="pdf")



# -*- coding: utf-8 -*-

__all__ = ["StellarPopulation"]

import os
import numpy as np

from ._fsps import driver
from .filters import FILTERS


class StellarPopulation(object):
    r"""
    This is the main interface to use when interacting with FSPS from Python.
    Most of the Fortran API is exposed through Python hooks with various
    features added for user friendliness. It is recommended to only
    instantiate one StellarPopulation object in a given program. When
    initializing, you can set any of the parameters of the system using keyword
    arguments. Below, you'll find a list of the options that you can include
    (with the comments taken directly from the `FSPS docs
    <https://github.com/cconroy20/fsps/blob/master/doc/MANUAL.pdf>`_). Unless
    otherwise noted, you can change these values later using the ``params``
    property—which is ``dict``-like.  For example:

    ::

        sp = StellarPopulation(imf_type=2, zcontinuous=1)
        sp.params["imf_type"] = 1
        sp.params["logzsol"] = -0.3
        sp.params["sfh"] = 1

    :param compute_vega_mags: (default: False)
        A switch that sets the zero points of the magnitude system: ``True``
        uses Vega magnitudes versus AB magnitudes. Can only be changed during
        initialization.

    :param vactoair_flag: (default: False)
        If ``True``, output wavelengths in air (rather than vac). Can only be
        changed during initialization.

    :param zcontinuous: (default: 0)
        Flag specifying how interpolation in metallicity of the simple stellar
        populations (SSPs) is performed before computing composite stellar
        population (CSP) models:

        * 0: No interpolation, use the metallicity index specified by ``zmet``.
        * 1: The SSPs are interpolated to the value of ``logzsol`` before the
          spectra and magnitudes are computed, and the value of ``zmet`` is
          ignored.
        * 2: The SSPs are convolved with a metallicity distribution function
          specified by the ``logzsol`` and ``pmetals`` parameters. The value of
          ``zmet`` is ignored.
        * 3: Use all available SSP metallicities when computing the composite
          model, for use exclusively with tabular SFHs where the metallicity
          evolution as function of age is given (see `set_tabular_sfh()`).  The
          values of ``zmet`` and ``logzsol`` are ignored.  Furthermore
          ``add_neb_emission`` must be set to False.

        Can only be changed during initialization.

    :param add_agb_dust_model: (default: True)
        Switch to turn on/off the AGB circumstellar dust model presented in
        Villaume (2014). NB: The AGB dust emission is scaled by the parameter
        `agb_dust`.

    :param add_dust_emission: (default: True)
        Switch to turn on/off the Draine & Li 2007 dust emission model.

    :param add_igm_absorption: (default: False)
        Switch to include IGM absorption via Madau (1995).  The ``zred``
        parameter must be non-zero for this switch to have any effect. The
        optical depth can be scaled using the ``igm_factor`` parameter.

    :param add_neb_emission: (default: False)
        Switch to turn on/off a nebular emission model (both continuum and line
        emission), based on Cloudy models from Nell Byler.  Contrary to FSPS,
        this option is turned off by default.

    :param add_neb_continuum: (default: True)
        Switch to turn on/off the nebular continuum component (automatically
        turned off if ``add_neb_emission`` is ``False``).

    :param add_stellar_remnants: (default: True)
        Switch to add stellar remnants in the stellar mass computation.

    :param redshift_colors: (default: False)
        Flag specifying how to compute magnitudes. This has no effect in
        python-FSPS. Magnitudes are always computed at a fixed redshift
        specified by ``zred`` or the ``redshift`` parameter of ``get_mags``.
        See `get_mags` for details.

    :param compute_light_ages: (default: False)
        Flag specifying whether to compute light- and mass-weighted ages.  If
        ``True`` then the returned spectra are actually light-weighted ages (in
        Gyr) at every wavelength, the returned magnitudes are filter
        transmission weighted averages of these, the ``log_lbol`` attribute is
        the bolometric luminosity weighted age, and the ``stellar_mass``
        attribute gives the mass-weighted age.

    :param nebemlineinspec: (default: True)
        Flag to include the emission line fluxes in the spectrum. Turning this off
        is a significant speedup in model calculation time. If not set, the line luminosities
        are still computed.

    :param smooth_velocity: (default: True)
        Switch to choose smoothing in velocity space (``True``) or wavelength
        space.

    :param smooth_lsf: (default: False)
        Switch to apply smoothing of the SSPs by a wavelength dependent line
        spread function. See the ``set_lsf()`` method for details.  Only takes
        effect if ``smooth_velocity`` is True.

    :param cloudy_dust: (default: False)
        Switch to include dust in the Cloudy tables.

    :param agb_dust: (default: 1.0)
        Scales the circumstellar AGB dust emission.

    :param tpagb_norm_type: (default: 2)
        Flag specifying TP-AGB normalization scheme:

        * 0: default Padova 2007 isochrones
        * 1: Conroy & Gunn 2010 normalization
        * 2: Villaume, Conroy, Johnson 2015 normalization

    :param dell: (default: 0.0)
        Shift in :math:`\log L_\mathrm{bol}` of the TP-AGB isochrones. Note
        that the meaning of this parameter and the one below has changed to
        reflect the updated calibrations presented in Conroy & Gunn (2009).
        That is, these parameters now refer to a modification about the
        calibrations presented in that paper.  Only has effect if
        ``tpagb_norm_type=1``.

    :param delt: (default: 0.0)
        Shift in :math:`\log T_\mathrm{eff}` of the TP-AGB isochrones.  Only
        has effect if ``tpagb_norm_type=1``.

    :param redgb: (default: 1.0)
        Modify weight given to RGB.  Only available with BaSTI isochrone set.

    :param agb: (default: 1.0)
        Modify weight given to TP-AGB.  This only has effect for FSPS v3.1 or
        higher.

    :param fcstar: (default: 1.0)
        Fraction of stars that the Padova isochrones identify as Carbon stars
        that FSPS assigns to a Carbon star spectrum. Set this to 0.0 if for
        example the users wishes to turn all Carbon stars into regular M-type
        stars.

    :param sbss: (default: 0.0)
        Specific frequency of blue straggler stars. See Conroy et al. (2009a)
        for details and a plausible range.

    :param fbhb: (default: 0.0)
        Fraction of horizontal branch stars that are blue. The blue HB stars
        are uniformly spread in :math:`\log T_\mathrm{eff}` to :math:`10^4`
        K. See Conroy et al. (2009a) for details and a plausible range.

    :param pagb: (default: 1.0)
        Weight given to the post–AGB phase. A value of 0.0 turns off post-AGB
        stars; a value of 1.0 implies that the Vassiliadis & Wood (1994) tracks
        are implemented as–is.

    :param zred: (default: 0.0)
        Redshift. If this value is non-zero and if ``redshift_colors=1``, the
        magnitudes will be computed for the spectrum placed at redshift
        ``zred``.

    :param zmet: (default: 1)
        The metallicity is specified as an integer ranging between 1 and nz. If
        ``zcontinuous > 0`` then this parameter is ignored.

    :param logzsol: (default: 0.0)
        Parameter describing the metallicity, given in units of :math:`\log
        (Z/Z_\odot)`.  Only used if ``zcontinuous > 0``.

    :param pmetals: (default: 2.0)
       The power for the metallicty distribution function.  The MDF is given by
       :math:`(Z \, e^{-Z})^{\mathrm{pmetals}}` where :math:`Z =
       z/(z_\odot \, 10^{\mathrm{logzsol}})` and z is the metallicity in
       linear units (i.e., :math:`z_\odot = 0.019`).  Using a negative value
       will result in smoothing of the SSPs by a three-point triangular kernel
       before linear interpolation (in :math:`\log Z`) to the requested
       metallicity. Only used if ``zcontinuous = 2``.

    :param imf_type: (default: 2)
        Common variable defining the IMF type:

        * 0: Salpeter (1955)
        * 1: Chabrier (2003)
        * 2: Kroupa (2001)
        * 3: van Dokkum (2008)
        * 4: Dave (2008)
        * 5: tabulated piece-wise power law IMF, specified in ``imf.dat`` file
          located in the data directory

    :param imf_upper_limit: (default: 120)
        The upper limit of the IMF, in solar masses. Note that if this is
        above the maximum mass in the isochrones then those stars will not
        contribute to the spectrum but will affect the overall IMF
        normalization.

    :param imf_lower_limit: (default: 0.08)
        The lower limit of the IMF, in solar masses.  Note that if this is
        below the minimum mass in the isochrones then those stars will not
        contribute to the spectrum but will affect the overall IMF
        normalization.

    :param imf1: (default: 1.3)
        Logarithmic slope of the IMF over the range :math:`0.08 < M < 0.5
        M_\odot`. Only used if ``imf_type=2``.

    :param imf2: (default: 2.3)
        Logarithmic slope of the IMF over the range :math:`0.5 < M < 1
        M_\odot`. Only used if ``imf_type=2``.

    :param imf3: (default: 2.3)
        Logarithmic slope of the IMF over the range :math:`1.0 < M < \mathrm{imf\_upper\_limit}
        M_\odot`. Only used if ``imf_type=2``.

    :param vdmc: (default: 0.08)
        IMF parameter defined in van Dokkum (2008). Only used if
        ``imf_type=3``.

    :param mdave: (default: 0.5)
        IMF parameter defined in Dave (2008). Only used if ``imf_type=4``.

    :param evtype: (default: -1)
        Compute SSPs for only the given evolutionary type. All phases used when
        set to -1.

    :param masscut: (default: 150.0)
        Truncate the IMF above this value.

    :param sigma_smooth: (default: 0.0)
        If smooth_velocity is True, this gives the velocity dispersion in km/s.
        Otherwise, it gives the width of the gaussian wavelength smoothing in
        Angstroms.  These widths are in terms of :math:`\sigma`, *not* FWHM.

    :param min_wave_smooth: (default: 1e3)
        Minimum wavelength to consider when smoothing the spectrum.

    :param max_wave_smooth: (default: 1e4)
        Maximum wavelength to consider when smoothing the spectrum.

    :param gas_logu: (default: -2)
        Log of the gas ionization parameter; relevant only for the nebular
        emission model.

    :param gas_logz: (default: 0.0)
        Log of the gas-phase metallicity; relevant only for the nebular
        emission model.  In units of :math:`\log (Z/Z_\odot)`.

    :param igm_factor: (default: 1.0)
        Factor used to scale the IGM optical depth.

    :param sfh: (default: 0)
        Defines the type of star formation history, normalized such that one
        solar mass of stars is formed over the full SFH. Default value is 0.

        * 0: Compute a simple stellar population (SSP).
        * 1: Tau-model. A six parameter SFH (tau model plus a constant
          component and a burst) with parameters ``tau``, ``const``,
          ``sf_start``, ``sf_trunc``, ``tburst``, and ``fburst`` (see below).
        * 2: This option is not supported in Python-FSPS.
        * 3: Compute a tabulated SFH, which is supplied through the
          ``set_tabular_sfh`` method.  See that method for details.
        * 4: Delayed tau-model. This is the same as option 1 except that the
          tau-model component takes the form :math:`t\,e^{−t/\tau}`.
        * 5: Delayed tau-model with a transition at a time ``sf_trunc`` to a
          linearly decreasing SFH with the slope specified by ``sf_slope``. See
          Simha et al. 2014 for details.

    :param tau: (default: 1.0)
        Defines e-folding time for the SFH, in Gyr. Only used if ``sfh=1`` or
        ``sfh=4``. The range is :math:`0.1 < \tau < 10^2`.

    :param const: (default: 0.0)
        Defines the constant component of the SFH. This quantity is defined as
        the fraction of mass formed in a constant mode of SF; the range is
        therefore :math:`0 \le C \le 1`. Only used if ``sfh=1`` or ``sfh=4``.

    :param sf_start: (default: 0.0)
        Start time of the SFH, in Gyr. Only used if ``sfh=1`` or ``sfh=4`` or
        ``sfh=5``.

    :param sf_trunc: (default: 0.0)
        Truncation time of the SFH, in Gyr. If set to 0.0, there is no
        trunction.  Only used if ``sfh=1`` or ``sfh=4`` or ``sfh=5``.

    :param tage: (default: 0.0)
        If set to a non-zero value, the
        :func:`fsps.StellarPopulation.compute_csp` method will compute the
        spectra and magnitudes only at this age, and will therefore only output
        one age result. The units are Gyr. (The default is to compute and
        return results from :math:`t \approx 0` to the maximum age in the
        isochrones).

    :param fburst: (default: 0.0)
        Deﬁnes the fraction of mass formed in an instantaneous burst of star
        formation. Only used if ``sfh=1`` or ``sfh=4``.

    :param tburst: (default: 11.0)
        Defines the age of the Universe when the burst occurs. If ``tburst >
        tage`` then there is no burst. Only used if ``sfh=1`` or ``sfh=4``.

    :param sf_slope: (default: 0.0)
        For ``sfh=5``, this is the slope of the SFR after time ``sf_trunc``.

    :param dust_type: (default: 0)
        Common variable deﬁning the extinction curve for dust around old stars:

        * 0: power law with index dust index set by ``dust_index``.
        * 1: Milky Way extinction law (with the :math:`R = A_V /E(B - V)` value
          given by ``mwr``) parameterized by Cardelli et al. (1989), with
          variable UV bump strength (see ``uvb`` below).
        * 2: Calzetti et al. (2000) attenuation curve. Note that if this value
          is set then the dust attenuation is applied to all starlight equally
          (not split by age), and therefore the only relevant parameter is
          ``dust2``, which sets the overall normalization (you must set
          ``dust1=0.0`` for this to work correctly).
        * 3: allows the user to access a variety of attenuation curve models
          from Witt & Gordon (2000) using the parameters ``wgp1`` and
          ``wgp2``. In this case the parameters ``dust1`` and ``dust2`` have no
          effect because the WG00 models specify the full attenuation curve.
        * 4: Kriek & Conroy (2013) attenuation curve.  In this model the slope
          of the curve, set by the parameter ``dust_index``, is linked to the
          strength of the UV bump.

    :param dust_tesc: (default: 7.0)
        Stars younger than ``dust_tesc`` are attenuated by both ``dust1`` and
        ``dust2``, while stars older are attenuated by ``dust2`` only. Units
        are :math:`\log (\mathrm{yrs})`.

    :param dust1: (default: 0.0)
        Dust parameter describing the attenuation of young stellar light,
        i.e. where ``t <= dust_tesc`` (for details, see Conroy et al. 2009a).

    :param dust2: (default: 0.0)
        Dust parameter describing the attenuation of old stellar light,
        i.e. where ``t > dust_tesc`` (for details, see Conroy et al. 2009a).

    :param dust_clumps: (default: -99.)
        Dust parameter describing the dispersion of a Gaussian PDF density
        distribution for the old dust. Setting this value to -99.0 sets the
        distribution to a uniform screen. See Conroy et al. (2009b) for
        details.  Values other than -99 are no longer supported.

    :param frac_nodust: (default: 0.0)
        Fraction of starlight that is not attenuated by the diffuse dust
        component (i.e. that is not affected by ``dust2``).

    :param frac_obrun: (default: 0.0)
        Fraction of the young stars (age < dust_tesc) that are not attenuated
        by ``dust1``, representing runaway OB stars.  These stars are still
        attenuated by ``dust2``.

    :param dust_index: (default: -0.7)
        Power law index of the attenuation curve. Only used when
        ``dust_type=0``.

    :param dust1_index: (default: -1.0)
        Power law index of the attenuation curve affecting stars younger than
        dust_tesc corresponding to ``dust1``. Used for all dust types.

    :param mwr: (default: 3.1)
        The ratio of total to selective absorption which characterizes the MW
        extinction curve: :math:`R = A_V /E(B - V)`. Only used when
        ``dust_type=1``.

    :param uvb: (default: 1.0)
        Parameter characterizing the strength of the 2175A extinction feature
        with respect to the standard Cardelli et al. determination for the
        MW. Only used when ``dust_type=1``.

    :param wgp1: (default: 1)
        Integer specifying the optical depth in the Witt & Gordon (2000)
        models. Values range from 1 − 18, corresponding to optical depths of
        0.25, 0.50, 0.75, 1.00, 1.50, 2.00, 2.50, 3.00, 3.50, 4.00, 4.50,
        5.00, 5.50, 6.00, 7.00, 8.00, 9.00, 10.0. Note that these optical
        depths are defined differently from the optical depths defined by
        the parameters ``dust1`` and ``dust2``. See Witt & Gordon (2000)
        for details.

    :param wgp2: (default: 1)
        Integer specifying the type of large-scale geometry and extinction
        curve. Values range from 1-6, corresponding to MW+dusty, MW+shell,
        MW+cloudy, SMC+dusty, SMC+shell, SMC+cloudy. Dusty, shell, and cloudy
        specify the geometry and are described in Witt & Gordon (2000).

    :param wgp3: (default: 1)
        Integer specifying the local geometry for the Witt & Gordon (2000)
        dust models. A value of 0 corresponds to a homogeneous distribution,
        and a value of 1 corresponds to a clumpy distribution. See Witt &
        Gordon (2000) for details.

    :param duste_gamma: (default: 0.01)
        Parameter of the Draine & Li (2007) dust emission model. Specifies the
        relative contribution of dust heated at a radiation field strength of
        :math:`U_\mathrm{min}` and dust heated at :math:`U_\mathrm{min} < U \le
        U_\mathrm{max}`. Allowable range is 0.0 – 1.0.

    :param duste_umin: (default: 1.0)
        Parameter of the Draine & Li (2007) dust emission model. Specifies the
        minimum radiation field strength in units of the MW value. Valid range
        is 0.1 – 25.0.

    :param duste_qpah: (default: 3.5)
        Parameter of the Draine & Li (2007) dust emission model. Specifies the
        grain size distribution through the fraction of grain mass in
        PAHs. This parameter has units of % and a valid range of 0.0 − 10.0.

    :param fagn: (default: 0.0)
        The total luminosity of the AGN, expressed as a fraction of the
        bolometric stellar luminosity (so it can be greater than 1). The shape
        of the AGN SED is from the Nenkova et al. 2008 templates.

    :param agn_tau: (default: 10)
        Optical depth of the AGN dust torus, which affects the shape of the AGN
        SED.  Outside the range (5, 150) the AGN SED is an
        extrapolation.
    """

    def __init__(
        self, compute_vega_mags=False, vactoair_flag=False, zcontinuous=0, **kwargs
    ):

        # Set up the parameters to their default values.
        self.params = ParameterSet(
            add_agb_dust_model=True,
            add_dust_emission=True,
            add_igm_absorption=False,
            add_neb_emission=False,
            add_neb_continuum=True,
            add_stellar_remnants=True,
            redshift_colors=False,
            compute_light_ages=False,
            nebemlineinspec=True,
            smooth_velocity=True,
            smooth_lsf=False,
            cloudy_dust=False,
            agb_dust=1.0,
            tpagb_norm_type=2,
            dell=0.0,
            delt=0.0,
            redgb=1.0,
            agb=1.0,
            fcstar=1.0,
            fbhb=0.0,
            sbss=0.0,
            pagb=1.0,
            zred=0.0,
            zmet=1,
            logzsol=0.0,
            pmetals=2.0,
            imf_type=2,
            imf_upper_limit=120,
            imf_lower_limit=0.08,
            imf1=1.3,
            imf2=2.3,
            imf3=2.3,
            vdmc=0.08,
            mdave=0.5,
            evtype=-1,
            masscut=150.0,
            sigma_smooth=0.0,
            min_wave_smooth=1e3,
            max_wave_smooth=1e4,
            gas_logu=-2,
            gas_logz=0.0,
            igm_factor=1.0,
            sfh=0,
            tau=1.0,
            const=0.0,
            sf_start=0.0,
            sf_trunc=0.0,
            tage=0.0,
            dust_tesc=7.0,
            fburst=0.0,
            tburst=11.0,
            sf_slope=0.0,
            dust_type=0,
            dust1=0.0,
            dust2=0.0,
            dust_clumps=-99.0,
            frac_nodust=0.0,
            frac_obrun=0.0,
            dust_index=-0.7,
            dust1_index=-1.0,
            mwr=3.1,
            uvb=1.0,
            wgp1=1,
            wgp2=1,
            wgp3=1,
            duste_gamma=0.01,
            duste_umin=1.0,
            duste_qpah=3.5,
            fagn=0.0,
            agn_tau=10.0,
        )

        # Parse any input options.
        for k, v in self.params.iteritems():
            self.params[k] = kwargs.pop(k, v)

        # Make sure that we didn't get any unknown options.
        if len(kwargs):
            raise TypeError(
                "__init__() got an unexpected keyword argument "
                "'{0}'".format(list(kwargs)[0])
            )

        # Before the first time we interact with the FSPS driver, we need to
        # run the ``setup`` method.
        if not driver.is_setup:
            driver.setup(compute_vega_mags, vactoair_flag)
        else:
            cvms, vtaflag = driver.get_setup_vars()
            assert compute_vega_mags == bool(cvms)
            assert vactoair_flag == bool(vtaflag)
        self._zcontinuous = zcontinuous
        # Caching.
        self._wavelengths = None
        self._emwavelengths = None
        self._zlegend = None
        self._ssp_ages = None
        self._stats = None
        self._libraries = None

    def _update_params(self):
        if self.params.dirtiness == 2:
            driver.set_ssp_params(*[self.params[k] for k in self.params.ssp_params])
        if self.params.dirtiness >= 1:
            driver.set_csp_params(*[self.params[k] for k in self.params.csp_params])
        self.params.dirtiness = 0

    def _compute_csp(self):
        self._update_params()

        NSPEC = driver.get_nspec()
        NTFULL = driver.get_ntfull()
        driver.compute_zdep(NSPEC, NTFULL, self._zcontinuous)

        self._stats = None

    def get_spectrum(self, zmet=None, tage=0.0, peraa=False):
        r"""
        Return spectra for the current CSP.

        :param zmet: (default: None)
            The (integer) index of the metallicity to use. By default, use
            the current value of ``self.params["zmet"]``.

        :param tage: (default: 0.0)
            The age of the stellar population for which to obtain a
            spectrum. By default, this will compute a grid of ages from
            :math:`t \approx 0` to the maximum age in the isochrones.

        :param peraa: (default: False)
            If ``True``, return the spectrum in :math:`L_\odot/A`. Otherwise,
            return the spectrum in the FSPS standard
            :math:`L_\odot/\mathrm{Hz}`.

        :returns wavelengths:
            The wavelength grid in Angstroms.

        :returns spectrum:
            The spectrum in :math:`L_\odot/\mathrm{Hz}` or :math:`L_\odot/A`.
            If an age was provided by the ``tage`` parameter then the result
            is a 1D array with ``NSPEC`` values. Otherwise, it is a 2D array
            with shape ``(NTFULL, NSPEC)``.
        """
        self.params["tage"] = tage
        if zmet is not None:
            self.params["zmet"] = zmet

        if self.params.dirty:
            self._compute_csp()

        wavegrid = self.wavelengths
        if peraa:
            factor = 3e18 / wavegrid ** 2

        else:
            factor = np.ones_like(wavegrid)

        NSPEC = driver.get_nspec()
        if (tage > 0.0) or (tage == -99):
            return wavegrid, driver.get_spec(NSPEC, 1)[0] * factor

        NTFULL = driver.get_ntfull()
        return wavegrid, driver.get_spec(NSPEC, NTFULL) * factor[None, :]

    def get_mags(self, zmet=None, tage=0.0, redshift=None, bands=None):
        r"""
        Get the magnitude of the CSP.

        :param zmet: (default: None)
            The (integer) index of the metallicity to use. By default, use the
            current value of ``self.params["zmet"]``.

        :param tage: (default: 0.0)
            The age of the stellar population. By default, this will compute a
            grid of ages from :math:`t \approx 0` to the maximum age in the
            isochrones.

        :param redshift: (default: None)
            Optionally redshift the spectrum first. If not supplied, the
            redshift given by ``StellarPopulation.params["zred"]`` is assumed.
            If supplied, the value of ``zred`` is ignored (and IGM attenuation
            will not work properly).

        :param bands: (default: None)
            The names of the filters that you would like to compute the
            magnitude for. This should correspond to the result of
            :func:`fsps.find_filter`.

        :returns mags:
            The magnitude grid. If an age was was provided by the ``tage``
            parameter then the result is a 1D array with ``NBANDS`` values.
            Otherwise, it is a 2D array with shape ``(NTFULL, NBANDS)``. If a
            particular set of bands was requested then this return value will
            be properly compressed along that axis, ordered according to the
            ``bands`` argument. If ``redshift`` is not 0, the units are
            apparent observed frame magnitude :math:`m` assuming
            :math:`\Omega_m=0.3, \Omega_\Lambda=0.7`
        """
        if redshift is None:
            zr = self.params["zred"]
        elif (self.params["zred"] > 0) & (redshift != self.params["zred"]):
            zr = redshift
            print("Warning: redshift is different than 'zred'.")
        else:
            zr = redshift
        self.params["tage"] = tage
        if zmet is not None:
            self.params["zmet"] = zmet

        if self.params.dirty:
            self._compute_csp()

        if tage > 0.0:
            NTFULL = 1
        else:
            NTFULL = driver.get_ntfull()
        NBANDS = driver.get_nbands()
        NSPEC = driver.get_nspec()
        band_array = np.ones(NBANDS, dtype=bool)
        if bands is not None:
            user_sorted_inds = np.array([FILTERS[band.lower()].index for band in bands])
            band_array[
                np.array(
                    [i not in user_sorted_inds for i in range(NBANDS)],
                    dtype=bool,
                )
            ] = False

        inds = np.array(band_array, dtype=int)
        mags = driver.get_mags(NSPEC, NTFULL, zr, inds)

        if tage > 0.0:
            if bands is not None:
                return mags[0, user_sorted_inds]
            else:
                return mags[0, :]
        else:
            if bands is not None:
                return mags[:, user_sorted_inds]
            else:
                return mags

    def _ztinterp(self, zpos, tpos, peraa=False):
        r"""
        Return an SSP spectrum, mass, and luminosity interpolated to a target
        metallicity and age.  This effectively wraps the ZTINTERP subroutine.
        Only the SSPs bracketing a given metallicity will be regenerated, if
        parameters are dirty.

        :param zpos:
            The metallicity, in units of :math:`\log(Z/Z_\odot)`

        :param tpos:
            The desired age, in Gyr.

        :param peraa: (default: False)
            If true, return spectra in units of :math:`L_\odot/A`, otherwise
            :math:`L_\odot/\mathrm{Hz}`

        :returns spec:
            The SSP spectrum, interpolated to zpos and tpos.

        :returns mass:
            The stellar mass of the SSP at tpos.

        :returns lbol:
            The bolometric luminosity of the returned SSP.
        """
        if self.params.dirtiness == 2:
            self._update_params()

        NSPEC = driver.get_nspec()
        spec, mass, lbol = np.zeros(NSPEC), np.zeros(1), np.zeros(1)
        logt_yrs = np.log10(tpos * 1e9)
        driver.interp_ssp(zpos, logt_yrs, spec, mass, lbol)

        if peraa:
            wavegrid = self.wavelengths
            factor = 3e18 / wavegrid ** 2
            spec *= factor

        return spec, mass, lbol

    def _all_ssp_spec(self, update=True, peraa=False):
        r"""
        Return the contents of the ssp_spec_zz array.

        :param update: (default: True)
            If True, forces an update of the SSPs if the ssp parameters have
            changed. Otherwise simply dumps the current contents of the
            ``ssp_spec_zz`` array.

        :param peraa: (default: False)
            If true, return spectra in units of :math:`L_\odot/A`, otherwise
            :math:`L_\odot/\mathrm{Hz}`

        :returns spec:
            The spectra of the SSPs, having shape (nspec, ntfull, nz).

        :returns mass:
            The mass of the SSPs, having shape (ntfull, nz).

        :returns lbol:
            The bolometric luminosity of the SSPs, having shape (ntfull, nz).
        """

        if (self.params.dirtiness == 2) and update:
            self._update_params()

        NSPEC = driver.get_nspec()
        NTFULL = driver.get_ntfull()
        NZ = driver.get_nz()
        spec = np.zeros([NSPEC, NTFULL, NZ], order="F")
        mass = np.zeros([NTFULL, NZ], order="F")
        lbol = np.zeros([NTFULL, NZ], order="F")
        driver.get_ssp_spec(spec, mass, lbol)

        if peraa:
            wavegrid = self.wavelengths
            factor = 3e18 / wavegrid ** 2
            spec *= factor[:, None, None]

        return spec, mass, lbol

    def _get_stellar_spectrum(
        self,
        mact,
        logt,
        lbol,
        logg,
        phase,
        comp,
        mdot=0,
        weight=1,
        zmet=None,
        peraa=True,
    ):
        r"""
        Get the spectrum of a star with a given set of physical parameters.
        This uses the metallicity given by the current value of ``zmet``.

        :param mact:
            Actual stellar mass (after taking into account mass loss).  Used to
            calculate surface gravity.

        :param logt:
            The log of the effective temperature.

        :param lbol:
            Stellar luminosity, in units of :math:`L_\odot`

        :param logg:
            Log of the surface gravity g.  Note that this variable is actually
            ignored, and logg is calculated internally using ``mact``,
            ``lbol``, and ``logt``.

        :param phase:
            The evolutionary phase, 0 through 6.

        :param comp:
            Composition, in terms of C/O ratio.  Only used for AGB stars
            (phase=5), where the division between carbon and oxyygen rich stars
            is :math:`C/O = 1`.

        :param mdot:
            The log of the mass loss rate.

        :param weight:
            The IMF weight

        :returns outspec:
            The spectrum of the star, in :math:`L_\odot/\mathrm{Hz}`
        """
        if zmet is not None:
            self.params["zmet"] = zmet

        if self.params.dirty:
            self._update_params()

        NSPEC = driver.get_nspec()
        outspec = np.zeros(NSPEC)
        driver.stellar_spectrum(
            mact, logt, lbol, logg, phase, comp, mdot, weight, outspec
        )
        if peraa:
            wavegrid = self.wavelengths
            factor = 3e18 / wavegrid ** 2
            outspec *= factor

        return outspec

    def isochrones(self, outfile="pyfsps_tmp"):
        r"""
        Write the isochrone data (age, mass, weights, phases, magnitudes, etc.)
        to a .cmd file, then read it into a huge numpy array. Only parameters
        listed in ``StellarPopulation.params.ssp_params`` affect the output of
        this method.

        :param outfile: (default: 'pyfsps_tmp')
            The file root name of the .cmd file, which will be placed in the
            $SPS_HOME/OUTPUTS/ directory

        :returns dat:
            A huge numpy structured array containing information about every
            isochrone point for the current metallicity.  In general the
            columns may be isochrone specific, but for Padova they are

            * age: log age, yrs
            * log(Z): log metallicity
            * mini: initial stellar mass in solar masses
            * mact: actual stellar mass (accounting for mass loss)
            * logl: log bolometric luminosity, solar units
            * logt: log temperature (K)
            * logg: log gravity
            * phase: (see evtype)
            * log(weight): IMF weight corresponding to a total of 1 Msol formed.
            * log(mdot): mass loss rate (Msol/yr)
        """
        if self.params.dirty:
            self._compute_csp()

        from . import list_filters

        absfile = os.path.join(os.environ["SPS_HOME"], "OUTPUTS", outfile + ".cmd")
        driver.write_isoc(outfile)

        with open(absfile, "r") as f:
            # drop the comment hash and mags field
            header = f.readline().split()[1:-1]
        header += list_filters()
        cmd_data = np.loadtxt(
            absfile,
            comments="#",
            dtype=np.dtype([(n, np.float) for n in header]),
        )
        return cmd_data

    def set_tabular_sfh(self, age, sfr, Z=None):
        r"""
        Set a tabular SFH for use with the ``sfh=3`` option.  See the FSPS
        documentation for information about tabular SFHs.  This SFH will be
        piecewise linearly interpolated.

        :param age:
            Time since the beginning of the universe in Gyr.  Must be
            increasing.  ndarray of shape (ntab,)

        :param sfr:
            The SFR at each ``age``, in Msun/yr.  Must be an ndarray same
            length as ``age``, and contain at least one non-zero value.

        :param Z: (optional)
            The metallicity at each age, in units of absolute metallicity
            (e.g. Z=0.019 for solar with the Padova isochrones and MILES
            stellar library).
        """
        assert len(age) == len(sfr), "age and sfr have different size."
        assert np.all(age[1:] > age[:-1]), "Ages must be increasing."
        assert np.any(sfr > 1e-33), "At least one sfr must be > 1e-33."
        assert np.all(sfr >= 0.0), "sfr cannot be negative."
        ntab = len(age)
        if Z is None:
            Z = np.zeros(ntab)
            assert self._zcontinuous != 3
        else:
            assert len(Z) == ntab
            assert np.all(Z >= 0), "All values of Z must be greater than or equal 0."
            assert self._zcontinuous == 3, "_zcontinuous must be 3 for multi-Z tabular."
            assert self.params["add_neb_emission"] is False, (
                "Cannot compute nebular emission " "with multi-metallicity tabular SFH."
            )

        driver.set_sfh_tab(age * 1e9, sfr, Z)
        if self.params["sfh"] == 3:
            self.params.dirtiness = max(1, self.params.dirtiness)
        else:
            print(
                "Warning: You are setting a tabular SFH, "
                "but the ``sfh`` parameter is not 3"
            )

    def set_lsf(self, wave, sigma, wmin=None, wmax=None):
        r"""
        Set a wavelength dependent Gaussian line-spread function that will be
        applied to the SSPs.  Only takes effect if ``smooth_lsf`` and
        ``smooth_velocity`` are True.

        :param wave:
            Wavelength in angstroms, sorted ascending.  If `wmin` or `wmax`
            are not specified they are taken from the minimum and maximum of
            this array.  ndarray.

        :param sigma:
            The dispersion of the Gaussian LSF at the wavelengths given by
            `wave`, in km/s.  If 0, no smoothing is applied at that wavelength.
            ndarray of same shape as `wave`.

        :param wmin: (optional)
            The minimum wavelength (in AA) for which smoothing will be
            applied. If not given, it is taken from the minimum of `wave`.

        :param wmax: (optional)
            The maximum wavelength (in AA) for which smoothing will be
            applied. If not given, it is taken from the maximum of `wave`.
        """

        if wmin is None:
            wmin = wave.min()
        if wmax is None:
            wmax = wave.max()
        sig = np.interp(self.wavelengths, wave, sigma)
        driver.set_ssp_lsf(sig, wmin, wmax)
        if self.params["smooth_lsf"]:
            self.params.dirtiness = max(2, self.params.dirtiness)
        else:
            print(
                "Warning: You are setting an LSF for the SSPs, "
                "but the ``smooth_lsf`` parameter is not True."
            )
        if not self.params["smooth_velocity"]:
            print(
                "Warning: You are setting an LSF for the SSPs, "
                "but the ``smooth_velocity`` parameter is not True."
            )

    def smoothspec(self, wave, spec, sigma, minw=None, maxw=None):
        r"""
        Smooth a spectrum by a gaussian with standard deviation given by sigma.
        Whether the smoothing is in velocity space or in wavelength space
        depends on the value of the value of smooth_velocity.

        :param wave:
            The input wavelength grid.

        :param spec:
            The input spectrum.

        :param sigma:
            The standard deviation of the gaussian broadening function.

        :param minw:
            Optionally set the minimum wavelength to consider when
            broadening.

        :param maxw:
            Optionally set the maximum wavelength to consider when
            broadening.

        :returns outspec:
            The smoothed spectrum, on the same wavelength grid as the input.
        """
        if maxw is None:
            maxw = np.max(wave)
        if minw is None:
            minw = np.min(wave)
        assert len(wave) == len(spec)
        outspec = np.array(spec)
        driver.smooth_spectrum(wave, outspec, sigma, minw, maxw)
        return outspec

    def filter_data(self):
        r"""
        Return effective wavelengths, and vega and solar magnitudes
        of all filters.

        :returns lambda_eff:
            Effective wavelength of each filter.

        :returns magvega:
            The AB magnitude of Vega (used to convert between AB and Vega
            systems).

        :returns magsun:
            The AB absolute magnitude of the Sun.
        """
        NBANDS = driver.get_nbands()
        lambda_eff, magvega, magsun = driver.get_filter_data(NBANDS)
        return lambda_eff, magvega, magsun

    @property
    def wavelengths(self):
        r"""The wavelength scale for the computed spectra, in Angstroms"""
        if self._wavelengths is None:
            NSPEC = driver.get_nspec()
            self._wavelengths = driver.get_lambda(NSPEC)
        return self._wavelengths.copy()

    @property
    def emline_wavelengths(self):
        r"""Emission line wavelengths, in Angstroms"""
        if self._emwavelengths is None:
            NLINE = driver.get_nemline()
            self._emwavelengths = driver.get_emlambda(NLINE)
        return self._emwavelengths.copy()

    @property
    def zlegend(self):
        r"""The available metallicities."""
        if self._zlegend is None:
            NZ = driver.get_nz()
            self._zlegend = driver.get_zlegend(NZ)
        return self._zlegend

    @property
    def ssp_ages(self):
        r"""The age grid of the SSPs, in log(years), used by FSPS."""
        if self._ssp_ages is None:
            NTFULL = driver.get_ntfull()
            self._ssp_ages = driver.get_timefull(NTFULL)
        return self._ssp_ages

    @property
    def log_age(self):
        r"""log10(age/yr)."""
        return self._stat(0)

    @property
    def stellar_mass(self):
        r"""Surviving stellar mass in solar masses (including remnants if the
        FSPS parameter ``add_stellar_remants=1``).
        """
        return self._stat(1)

    @property
    def log_lbol(self):
        r"""log(bolometric luminosity / :math:`L_\odot`)."""
        return self._stat(2)

    @property
    def sfr(self):
        r"""Star formation rate (:math:`M_\odot/yr`)."""
        return self._stat(3)

    @property
    def dust_mass(self):
        r"""Dust mass, in solar masses."""
        return self._stat(4)

    @property
    def formed_mass(self):
        r"""Integral of the SFH, in solar masses."""
        return self._stat(5)

    @property
    def emline_luminosity(self):
        r"""emission line luminosities, in :math:`L_\odot`. shape=(ne)"""
        return self._stat(6)

    def _get_grid_stats(self):
        if self.params.dirty:
            self._compute_csp()

        if self._stats is None:
            self._stats = driver.get_stats(driver.get_ntfull(), driver.get_nemline())

        return self._stats

    def _stat(self, k):
        stats = self._get_grid_stats()
        if self.params["tage"] > 0:
            return stats[k][0]
        return stats[k]

    def sfr_avg(self, times=None, dt=0.1):
        r"""
        The average SFR between ``time``-``dt`` and ``time``, given the
        SFH parameters, for ``sfh=1`` or ``sfh=4``.  Like SFHSTAT in FSPS.
        Requires scipy, as it uses gamma functions.

        :param times: (default, None)
            The times (in Gyr of cosmic time) at which the average SFR over the
            last ``dt`` is desired.  if ``None``, uses the current value of the
            ``tage`` in the parameter set. Scalar or iterable.

        :param dt: (default: 0.1)
            The time interval over which the recent SFR is averaged, in Gyr.
            defaults to 100 Myr (i.e. sfr8).

        :returns sfr_avg:
            The SFR at ``tage`` averaged over the last ``dt`` Gyr, in units of
            solar masses per year, such that :math:`1 M_\odot` formed by
            ``tage``.  Same shape as ``times``.  For ``times < sf_start + dt``
            the value of ``sfr_avg`` is ``nan``, for ``times > tage`` the value
            is 0.
        """
        from scipy.special import gammainc

        assert self.params["sf_trunc"] <= 0, "sfr_avg not supported for sf_trunc > 0"
        if self.params["sfh"] == 1:
            power = 1
        elif self.params["sfh"] == 4:
            power = 2
        else:
            raise ValueError("sfr_avg not supported for this SFH type.")

        tau, sf_start = self.params["tau"], self.params["sf_start"]
        if self.params["tage"] <= 0:
            tage = 10 ** (np.max(self.ssp_ages) - 9)
        else:
            tage = np.array(self.params["tage"])

        if times is None:
            times = tage
        else:
            times = np.array(times)

        tb = (self.params["tburst"] - sf_start) / tau
        tmax = (tage - sf_start) / tau
        normalized_times = (np.array([times, times - dt]).T - sf_start) / tau

        tau_mass_frac = gammainc(power, normalized_times) / gammainc(power, tmax)
        burst_in_past = tb <= normalized_times
        mass = (
            tau_mass_frac
            * (1 - self.params["const"] - (tb < tmax) * self.params["fburst"])
            + self.params["const"] * normalized_times / tmax
            + burst_in_past * self.params["fburst"]
        )

        avsfr = (mass[..., 0] - mass[..., 1]) / dt / 1e9  # Msun/yr

        # These lines change behavior when you request sfrs outside the range (sf_start + dt, tage)
        # avsfr[times > tage] = np.nan  # does not work for scalars
        avsfr *= times <= tage
        # avsfr[np.isfinite(avsfr)] = 0.0 # does not work for scalars

        return np.clip(avsfr, 0, np.inf)

    @property
    def isoc_library(self):
        r"""The name of the stellar isochrone library being used in FSPS."""
        return self.libraries[0]

    @property
    def spec_library(self):
        r"""The name of the stellar spectral library being used in FSPS."""
        return self.libraries[1]

    @property
    def duste_library(self):
        r"""The name of the dust emission SED library being used in FSPS."""
        return self.libraries[2]


    @property
    def libraries(self):
        if self._libraries is None:
            self._libraries = driver.get_libraries()
        return self._libraries


class ParameterSet(object):

    ssp_params = [
        "imf_type",
        "imf_upper_limit",
        "imf_lower_limit",
        "imf1",
        "imf2",
        "imf3",
        "vdmc",
        "mdave",
        "dell",
        "delt",
        "sbss",
        "fbhb",
        "pagb",
        "add_stellar_remnants",
        "tpagb_norm_type",
        "add_agb_dust_model",
        "agb_dust",
        "redgb",
        "agb",
        "masscut",
        "fcstar",
        "evtype",
        "smooth_lsf",
    ]

    csp_params = [
        "smooth_velocity",
        "redshift_colors",
        "compute_light_ages",
        "nebemlineinspec",
        "dust_type",
        "add_dust_emission",
        "add_neb_emission",
        "add_neb_continuum",
        "cloudy_dust",
        "add_igm_absorption",
        "zmet",
        "sfh",
        "wgp1",
        "wgp2",
        "wgp3",
        "tau",
        "const",
        "tage",
        "fburst",
        "tburst",
        "dust1",
        "dust2",
        "logzsol",
        "zred",
        "pmetals",
        "dust_clumps",
        "frac_nodust",
        "dust_index",
        "dust_tesc",
        "frac_obrun",
        "uvb",
        "mwr",
        "dust1_index",
        "sf_start",
        "sf_trunc",
        "sf_slope",
        "duste_gamma",
        "duste_umin",
        "duste_qpah",
        "sigma_smooth",
        "min_wave_smooth",
        "max_wave_smooth",
        "gas_logu",
        "gas_logz",
        "igm_factor",
        "fagn",
        "agn_tau",
    ]

    @property
    def all_params(self):
        return self.ssp_params + self.csp_params

    @property
    def dirty(self):
        return self.dirtiness > 0

    def __init__(self, **kwargs):
        self.dirtiness = 2
        self._params = kwargs
        try:
            self.iteritems = self._params.iteritems
        except AttributeError:
            self.iteritems = self._params.items

    def check_params(self):
        NZ = driver.get_nz()
        assert self._params["zmet"] in range(
            1, NZ + 1
        ), "zmet={0} out of range [1, {1}]".format(self._params["zmet"], NZ)
        assert self._params["dust_type"] in range(
            5
        ), "dust_type={0} out of range [0, 4]".format(self._params["dust_type"])
        assert self._params["imf_type"] in range(
            6
        ), "imf_type={0} out of range [0, 5]".format(self._params["imf_type"])
        assert (self._params["tage"] <= 0) | (
            self._params["tage"] > self._params["sf_start"]
        ), "sf_start={0} is greater than tage={1}".format(
            self._params["sf_start"], self._params["tage"]
        )
        assert (
            self._params["const"] + self._params["fburst"]
        ) <= 1, "const + fburst > 1"

    def __getitem__(self, k):
        return self._params[k]

    def __setitem__(self, k, v):
        original = self._params[k]
        is_changed = original != v

        if is_changed:
            if k in self.ssp_params:
                self.dirtiness = 2
            elif k in self.csp_params:
                self.dirtiness = max(1, self.dirtiness)

            self._params[k] = v
            self.check_params()



#!/usr/bin/env python
# encoding: utf-8
"""
Print a table of filters implemented by FSPS.

The table is formatted in restructured text for use with the python-fsps's
Sphinx documentation.
"""

from __future__ import print_function, absolute_import

import re
import fsps

# Pattern via https://gist.github.com/uogbuji/705383
URL_P = re.compile(ur'(?i)\b((?:https?://|www\d{0,3}[.]|[a-z0-9.\-]+[.][a-z]{2,4}/)(?:[^\s()<>]+|\(([^\s()<>]+|(\([^\s()<>]+\)))*\))+(?:\(([^\s()<>]+|(\([^\s()<>]+\)))*\)|[^\s`!()\[\]{};:\'".,<>?\xab\xbb\u201c\u201d\u2018\u2019]))')  # NOQA


def main():
    colnames = ("ID", "Name", "M_sun Vega", "M_sun AB", "lambda_eff (Å)",
                "Description")
    filters = [fsps.get_filter(name) for name in fsps.list_filters()]
    filter_list = make_filter_list(filters)
    txt = make_table(filter_list, colnames)
    print(txt)


def make_filter_list(filters):
    """Transform filters into list of table rows."""
    filter_list = []
    filter_ids = []
    for f in filters:
        filter_ids.append(f.index)
        fullname = URL_P.sub(r'`<\1>`_', f.fullname)
        filter_list.append((str(f.index + 1),
                            f.name,
                            "{0:.2f}".format(f.msun_vega),
                            "{0:.2f}".format(f.msun_ab),
                            "{0:.1f}".format(f.lambda_eff),
                            fullname))
    sortf = lambda item: int(item[0])
    filter_list.sort(key=sortf)
    return filter_list


def make_table(data, col_names):
    """Code for this RST-formatted table generator comes from
    http://stackoverflow.com/a/11350643
    """
    n_cols = len(data[0])
    assert n_cols == len(col_names)
    col_sizes = [max(len(r[i]) for r in data) for i in range(n_cols)]
    for i, cname in enumerate(col_names):
        if col_sizes[i] < len(cname):
            col_sizes[i] = len(cname)
    formatter = ' '.join('{:<%d}' % c for c in col_sizes)
    rows = '\n'.join([formatter.format(*row) for row in data])
    header = formatter.format(*col_names)
    divider = formatter.format(*['=' * c for c in col_sizes])
    output = '\n'.join((divider, header, divider, rows, divider))
    return output


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

__all__ = ["check_sps_home"]

import os


def check_sps_home():
    if "SPS_HOME" not in os.environ:
        raise RuntimeError("You need to have the SPS_HOME environment variable")

    path = os.environ["SPS_HOME"]
    if not os.path.isdir(path):
        raise RuntimeError(f"SPS_HOME environment variable '{path}' is not a directory")

    if not os.path.exists(os.path.join(path, "data", "emlines_info.dat")):
        raise RuntimeError(
            f"The FSPS directory at {path} doesn't seem to have the right data "
            "files installed"
        )



# -*- coding: utf-8 -*-

import os
import sys
import subprocess

import numpy as np
import pytest
from numpy.testing import assert_allclose

from fsps import StellarPopulation, filters


def test_fortran_error():
    output = subprocess.check_output(
        [
            sys.executable,
            os.path.join(
                os.path.dirname(os.path.abspath(__file__)), "simple_test_script.py"
            ),
        ]
    )
    assert output.decode("ascii").strip() == "success"


@pytest.fixture(scope="module")
def pop_and_params():
    pop = StellarPopulation(zcontinuous=1)
    params = dict([(k, pop.params[k]) for k in pop.params.all_params])
    return pop, params


def _reset_default_params(pop, params):
    pop._zcontinuous = 1
    for k in pop.params.all_params:
        pop.params[k] = params[k]


def _get_model(pop, theta):
    pop.params["imf3"] = theta
    assert pop.params.dirtiness == 2
    return pop.get_spectrum(tage=0.2)[1]


# def test_imf3_multiprocessing():
#     from multiprocessing import Pool
#     pool = Pool()
#     thetas = np.linspace(2.3, 8.3, 4)
#     single = map(_get_model, thetas)
#     multi = pool.map(_get_model, thetas)
#     assert_allclose(single, multi)


def test_libraries(pop_and_params):
    """This does not require or build clean SSPs"""
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    ilib, splib, dlib = pop.libraries
    assert ilib == pop.isoc_library
    assert splib == pop.spec_library
    assert dlib == pop.duste_library


def test_filters():
    """Test all the filters got transmission data loaded."""
    flist = filters.list_filters()
    # force trans cache to load
    filters.FILTERS[flist[0]]._load_transmission_cache()
    for f in flist:
        assert f in filters.TRANS_CACHE, "transmission not loaded for {}".format(f)


def test_get_mags(pop_and_params):
    """Basic test for supplying filter names to get_mags"""
    pop, params = pop_and_params
    fuv1 = pop.get_mags(bands=["galex_fuv"])[:, 0]
    mags = pop.get_mags()
    fuv2 = mags[:, 61]
    fuv3 = mags[:, 62]
    assert np.all(fuv1 == fuv2)
    assert np.all(fuv1 != fuv3)


def test_ssp(pop_and_params):
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    pop.params["sfh"] = 0
    wave, spec = pop.get_spectrum(tage=1, peraa=True)
    assert (wave[0] > 0) & (wave[0] < wave[-1]) & (wave[0] < 912.0)
    assert (wave[-1] > 1e6) & (wave[-1] < 1e10)
    Mv = 4.62  # AB absolute magnitude for a Zsol 1Gyr old SSP
    # This also tests get_mags
    mag = pop.get_mags(tage=1, bands=["v"])
    assert np.all(abs(mag - Mv) < 1.0)
    assert np.all((pop.stellar_mass < 1.0) & (pop.stellar_mass > 0))
    assert pop.params.dirtiness == 0


def test_csp_dirtiness(pop_and_params):
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    pop.params["sfh"] = 1
    pop.params["tau"] = 1.0
    wave, spec = pop.get_spectrum(tage=1.0)
    assert pop.params.dirtiness == 0
    pop.params["tau"] = 3.0
    assert pop.params.dirtiness == 1


def test_redshift(pop_and_params):
    """Test redshifting, make sure that
    1. redshifting does not persist in cached arrays
    2. specifying redshift via get_mags keyword or param key are consistent
    """
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    pop.params["sfh"] = 0
    pop.params["zred"] = 0.0
    pop.params["add_igm_absorption"] = False
    v1 = pop.get_mags(redshift=1.0, tage=1.0, bands=["v"])
    v2 = pop.get_mags(redshift=1.0, tage=1.0, bands=["v"])
    assert np.all(v1 == v2)

    pop.params["zred"] = 1.0
    v3 = pop.get_mags(redshift=None, tage=1.0, bands=["v"])
    v4 = pop.get_mags(redshift=None, tage=1.0, bands=["v"])
    v5 = pop.get_mags(redshift=0.0, tage=1.0, bands=["v"])
    assert np.all(v3 == v4)

    assert np.all(v3 == v1)
    assert np.all(v5 != v4)


def test_nebemlineinspec(pop_and_params):
    """Make sure nebular lines are actually added."""
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    pop.params["sfh"] = 4
    pop.params["tau"] = 5.0
    pop.params["add_neb_emission"] = True
    pop.params["nebemlineinspec"] = False
    wave, spec_neboff = pop.get_spectrum(tage=1.0)
    pop.params["nebemlineinspec"] = True
    wave, spec_nebon = pop.get_spectrum(tage=1.0)
    assert (spec_nebon - spec_neboff).sum() > 0
    assert np.all(np.isfinite(pop.emline_luminosity))
    assert np.all(np.isfinite(pop.emline_wavelengths))
    ha_idx = (wave > 6556) & (wave < 6573)
    assert (spec_nebon - spec_neboff)[ha_idx].sum() > 0


def test_mformed(pop_and_params):
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    pop.params["sfh"] = 1
    pop.params["const"] = 0.5
    w, s = pop.get_spectrum(tage=0)
    assert pop.formed_mass[-1] == 1
    assert pop.formed_mass[50] < 1.0
    assert pop.formed_mass[50] > 0.0
    w, s = pop.get_spectrum(tage=0)
    assert pop.formed_mass[-1] == 1.0


def test_light_ages(pop_and_params):
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    tmax = 5.0
    pop.params["sfh"] = 1
    pop.params["const"] = 0.5
    w, spec = pop.get_spectrum(tage=tmax)
    mstar = pop.stellar_mass
    lbol = pop.log_lbol
    pop.params["compute_light_ages"] = True
    w, light_age = pop.get_spectrum(tage=tmax)
    assert np.all(np.abs(np.log10(spec / light_age)) > 1)
    # make sure fuv really from young stars
    fuv = (w > 1220) & (w < 2000)
    assert (light_age[fuv]).max() < 0.1
    assert (light_age[fuv]).max() > 1e-5
    assert pop.log_lbol != lbol
    assert pop.stellar_mass != mstar
    assert pop.stellar_mass < tmax
    # luminosity weighted age always less than mass-weighted age
    # assert pop.log_lbol < pop.stellar_mass


def test_smoothspec(pop_and_params):
    # FIXME: This is not very stringent
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    wave, spec = pop.get_spectrum(tage=1, peraa=True)
    spec2 = pop.smoothspec(wave, spec, 160.0, minw=1e3, maxw=1e4)
    assert (spec - spec2 == 0.0).sum() > 0


def test_imf3(pop_and_params):
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    pop.params["imf_type"] = 2
    pop.params["imf3"] = 2.3
    w, model1 = pop.get_spectrum(tage=0.2)

    # check that changing the IMF does something
    pop.params["imf3"] = 8.3
    assert pop.params.dirtiness == 2
    w, model2 = pop.get_spectrum(tage=0.2)
    assert not np.allclose(model1 / model2 - 1.0, 0.0)

    # Do we *really* need to do this second check?
    pop.params["imf3"] = 2.3
    assert pop.params.dirtiness == 2
    w, model1b = pop.get_spectrum(tage=0.2)
    assert pop.params.dirtiness == 0

    assert_allclose(model1 / model1b - 1.0, 0.0)


def test_tabular(pop_and_params):
    pop, params = pop_and_params
    _reset_default_params(pop, params)

    import os

    fn = os.path.join(os.environ["SPS_HOME"], "data/sfh.dat")
    age, sfr, z = np.genfromtxt(fn, unpack=True, skip_header=0)

    # Mono-metallicity
    pop.params["sfh"] = 3
    pop.set_tabular_sfh(age, sfr)
    w, spec = pop.get_spectrum(tage=0)
    pop.set_tabular_sfh(age, sfr)
    assert pop.params.dirty
    w, spec = pop.get_spectrum(tage=0)
    assert spec.shape[0] == len(pop.ssp_ages)
    assert pop.params["sfh"] == 3
    w, spec_last = pop.get_spectrum(tage=-99)
    assert spec_last.ndim == 1
    w, spec = pop.get_spectrum(tage=age.max())
    assert np.allclose(spec / spec_last - 1.0, 0.0)
    pop.params["logzsol"] = -1
    w, spec_lowz = pop.get_spectrum(tage=age.max())
    assert not np.allclose(spec / spec_lowz - 1.0, 0.0)

    # Multi-metallicity
    pop._zcontinuous = 3
    pop.set_tabular_sfh(age, sfr, z)
    w, spec_multiz = pop.get_spectrum(tage=age.max())
    assert not np.allclose(spec_lowz / spec_multiz - 1.0, 0.0)

    pop._zcontinuous = 1
    pop.set_tabular_sfh(age, sfr)
    # get mass weighted metallicity
    mbin = np.gradient(age) * sfr
    mwz = (z * mbin).sum() / mbin.sum()
    pop.params["logzsol"] = np.log10(mwz / 0.019)
    w, spec_onez = pop.get_spectrum(tage=age.max())
    assert not np.allclose(spec_onez / spec_multiz - 1.0, 0.0)


def test_isochrones(pop_and_params):
    # Just test that `isochrones()` method runs
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    pop.params["imf_type"] = 0
    iso = pop.isochrones()


def test_smooth_lsf(pop_and_params):
    pop, params = pop_and_params
    _reset_default_params(pop, params)
    tmax = 1.0
    wave_lsf = np.arange(4000, 7000.0, 10)
    x = (wave_lsf - 5500) / 1500.0
    # a quadratic lsf dependence that goes from ~50 to ~100 km/s
    sigma_lsf = 50 * (1.0 + 0.4 * x + 0.6 * x ** 2)
    w, spec = pop.get_spectrum(tage=tmax)
    pop.params["smooth_lsf"] = True
    assert pop.params.dirtiness == 2
    pop.set_lsf(wave_lsf, sigma_lsf)
    w, smspec = pop.get_spectrum(tage=tmax)
    hi = w > 7100
    sm = (w < 7000) & (w > 3000)
    assert np.allclose(spec[hi] / smspec[hi] - 1.0, 0.0)
    assert not np.allclose(spec[sm] / smspec[sm] - 1.0, 0.0)
    pop.set_lsf(wave_lsf, sigma_lsf * 2)
    assert pop.params.dirtiness == 2


# Requires scipy
# def test_sfr_avg():

#    _reset_default_params()
#    pop.params['sfh'] = 1.0
#    pop.params['const'] = 0.5
#    w, spec = pop.get_spectrum(tage=0)
#    sfr6 = pop.sfr_avg(dt=1e-3)
#    dsfr = np.log10(pop.sfr/pop.sfr6)
#    good = pop.ssp_age > 6
#    assert np.all(np.abs(dsfr[good]) < 1e-2)



# -*- coding: utf-8 -*-

__all__ = [
    "__version__",
    "StellarPopulation",
    "find_filter",
    "get_filter",
    "list_filters",
]

# Check the that SPS_HOME variable is set properly
from .sps_home import check_sps_home  # noqa

check_sps_home()

# Finally actually import the functions
try:
    from .fsps_version import version as __version__  # noqa
except ImportError:
    __version__ = "develop"
from .fsps import StellarPopulation  # noqa
from .filters import find_filter, get_filter, list_filters  # noqa



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from __future__ import (division, print_function, absolute_import,
                        unicode_literals)
import pyfits
import numpy as np
import matplotlib.pyplot as pl
import fsps

# Measurements of cluster parameters.
tage = 10. ** (8.04 - 9)
logmass = 4.09
dist_mod = 24.5

# Set up the stellar population model.
sp = fsps.StellarPopulation(imf_type=2, dust_type=1, mwr=3.1, dust2=0.3)

# The measured magnitudes from the literature.
data = {"wfc3_f160w": 16.386,
        "wfc3_f275w": 17.398,
        "wfc_acs_f814w": 17.155}

# There are also a few filters that we have data for but aren't included in
# the standard FSPS install:
#   "F110W": 16.877,
#   "F336W": 17.349,
#   "F475W": 17.762,

# Load the observed spectrum.
f = pyfits.open("DAO69.fits")
obs_spec = np.array(f[0].data, dtype=float)
f.close()

obs_spec /= 5e-20

# The observed wavelength grid in the data is magically this:
obs_lambda = np.arange(0, 4540) * 1.2 + 3700

# Compute the model magnitudes.
for b, v in data.iteritems():
    print(b, v, sp.get_mags(zmet=20, tage=tage, band=b) - 2.5 * logmass
          + dist_mod)

# Compute the model spectrum in ``L_sun / A``.
lam, spec = sp.get_spectrum(zmet=20, tage=tage, peraa=True)
spec *= 3.839e33 * 10. ** (logmass - dist_mod / 2.5)

f = 1.0  # obs_spec[0, obs_lambda < 5000.][-1] / spec[lam < 5000.][-1]
print(obs_spec[0, obs_lambda < 5000.][-1] / spec[lam < 5000.][-1])

pl.loglog(obs_lambda, obs_spec[0], "k")
pl.loglog(lam, spec * f, "r")

pl.xlim(3700, 7000)
# pl.ylim(10 ** 3, 10 ** 4.5)
pl.savefig("spectrum.png")



#!/usr/bin/env python
# -*- coding: utf-8 -*-

"""feature_demo.py - This script creates a set of PDFs that illustrate the
effect on the SED of successively turning on various options or changing the
value of some variables.
"""

import os
import numpy as np
import matplotlib.pyplot as pl
from matplotlib.backends.backend_pdf import PdfPages

import fsps


def makefig(sps, tage=13.7, oldspec=None, **plotkwargs):
    w, spec = sps.get_spectrum(tage=tage)
    fig, ax = pl.subplots()
    if oldspec is not None:
        ax.plot(w, oldspec / w * 1e19, color='gray', linewidth=2, alpha=0.5)
    ax.plot(w, spec / w * 1e19, 'C2', linewidth=2)
    return fig, ax, spec


def prettify(fig, ax, label=None):
    ax.set_xlim(0.9e3, 1e6)
    ax.set_xscale('log')
    ax.set_ylim(0.01, 2)
    #ax.set_yscale('log')
    ax.set_xlabel(r'rest-frame $\lambda$ ($\AA$)', fontsize=20)
    ax.set_ylabel(r'$\lambda \, f_\lambda$', fontsize=20)
    ax.tick_params(axis='both', which='major', labelsize=16)
    if label is not None:
        ax.text(0.63, 0.85, label, transform=ax.transAxes, fontsize=16)

    fig.tight_layout()
    return fig, ax


if __name__ == "__main__":

    pl.rc('text', usetex=True)
    pl.rc('font', family='serif')
    pl.rc('axes', grid=False)
    pl.rc('xtick', direction='in')
    pl.rc('ytick', direction='in')
    pl.rc('xtick', top=True)
    pl.rc('ytick', right=True)

    sps = fsps.StellarPopulation(zcontinuous=1)
    ilib, slib, dlib = sps.libraries
    print(ilib, slib)
    os.makedirs("./figures", exist_ok=True)
    pdf = PdfPages('./figures/features.pdf')

    # Basic spectrum
    sps.params['sfh'] = 4
    sps.params['tau'] = 5.0
    sps.params['logzsol'] = 0.0
    sps.params['dust_type'] = 4  # kriek and Conroy
    sps.params['imf_type'] = 2  # kroupa
    sps.params['imf3'] = 2.3
    fig, ax, spec = makefig(sps)
    fig, ax = prettify(fig, ax, label=r"$\tau=5$, Age$=13.7$,\\n$\log Z/Z_\odot=0.0$")
    pdf.savefig(fig)
    pl.close(fig)

    # change IMF
    sps.params['imf3'] = 2.5
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"IMF slope")
    pdf.savefig(fig)

    # Attenuate
    sps.params['add_dust_emission'] = False
    sps.params['dust2'] = 0.2
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"Dust Attenuation")
    pdf.savefig(fig)
    pl.close(fig)

    # Dust emission
    sps.params['add_dust_emission'] = True
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"Dust Emission")
    pdf.savefig(fig)
    pl.close(fig)

    # Dust temperature
    sps.params['duste_umin'] = 10
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"Dust SED\\n({})".format(dlib))
    pdf.savefig(fig)
    pl.close(fig)

    # AGN emission
    sps.params['fagn'] = 0.3
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"AGN dust\\n(Nenkova)")
    pdf.savefig(fig)
    pl.close(fig)

    # Nebular emission
    sps.params['add_neb_emission'] = True
    sps.params['gas_logu'] = -3.5
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"Neb. emission\\n(Byler)")
    pdf.savefig(fig)
    pl.close(fig)

    # change logu
    sps.params['gas_logu'] = -1.0
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"Change U$_{neb}$")
    pdf.savefig(fig)
    pl.close(fig)

    # change logz
    sps.params['logzsol'] = -0.5
    sps.params['gas_logz'] = -0.5
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"$\log Z/Z_\odot=-0.5$")
    pdf.savefig(fig)
    pl.close(fig)

    # IGM absorption
    sps.params['zred'] = 6.0
    sps.params['add_igm_absorption'] = True
    fig, ax, spec = makefig(sps, oldspec=spec)
    fig, ax = prettify(fig, ax, label=r"IGM attenuation\\n(Madau, $z=6$)")
    pdf.savefig(fig)
    pl.close(fig)

    pdf.close()



# This demo shows how to make a plot of the fractional contribution of differnt
# stellar mass ranges to the total spectrum, for different properties of the
# stellar population.  There is also some use of the filter objects.

from itertools import product
import numpy as np
import matplotlib.pyplot as pl
import fsps

# make an SPS object with interpolation in metallicity enabled, and
# set a few parameters
sps = fsps.StellarPopulation(zcontinuous=1)
sps.params['imf_type'] = 0   # Salpeter IMF
sps.params['sfh'] = 1  # Tau model SFH

# Get a few filter transmission curves
filterlist = {'galex_fuv': 'FUV', 'galex_nuv': 'NUV', 'sdss_u': 'u',
              'sdss_g': 'g', 'sdss_i': 'i', '2mass_j': 'J'}
filters = [fsps.filters.get_filter(f) for f in filterlist]


# This is the main function
def specplots(tage=10.0, const=1.0, tau=1.0, neb=False, z=0.0, savefig=True,
              masslims=[1.0, 3.0, 15.0, 30.0, 120.0], **kwargs):
    """Make a number of plots showing the fraction of the total light due to
    stars within certain stellar mass ranges.

    :param tage:
        The age of the stellar population

    :param const:
        0 or 1.  fraction of SF in constant SFR fraction

    :param tau:
        If const is 0, this is the decay timescale (in Gyr) of the
        exponentially declining SFR.

    :param neb:
        Boolean.  Whether to include the contribution of emission lines.

    :param z:
        Metallicity, in units of log Z/Z_\odot

    :param masslims:
        The edges of the stellar mass bins to plot, in solar masses.

    :param savefig:
        If True save the figures to pdfs

    :returns cfig:
        matplotlib figure giving a plot of the spectrum of stars *less than* a
        given upper stellar mass limit

    :returns dfig:
        matplotlib figure giving a plot of the spectrum of stars in given mass
        ranges.

    :returns ffig:
        matplotlib figure giving a plot of the fraction of total flux due to
        stars in given mass ranges, as a function of wavelength.
    """
    # Set the FSPS params
    sps.params['const'] = const
    sps.params['tau'] = tau
    sps.params['add_neb_emission'] = neb
    sps.params['masscut'] = 120
    sps.params['logzsol'] = z
    # Try to set any extra params
    for k, v in kwargs.items():
        try:
            sps.params[k] = v
        except(KeyError):
            pass

    # Make a pretty string with the FSPS parameters
    sfh = {1: 'Constant SFR', 0: r'$\tau_{{SF}}={}$'.format(sps.params['tau'])}
    sfhname = {1: 'const', 0: 'tau{}'.format(sps.params['tau'])}
    name = '{}_z{}_neb{}'.format(sfhname[int(const)], z, neb)

    # get the total spectrum due to *all* stars
    wave, spec_tot = sps.get_spectrum(tage=tage)

    # Set up stellar mass ranges and arrays to hold output spectra
    mspec = np.zeros([len(sps.wavelengths), len(masslims)])
    dmspec = np.zeros_like(mspec)

    # set up figure and axis objects
    cfig, cax = pl.subplots(figsize=(12.4, 7.5))
    dfig, dax = pl.subplots(figsize=(12.4, 7.5))
    ffig, fax = pl.subplots(figsize=(12.4, 7.5))
    [ax.set_xlim(0.9e3, 1e4) for ax in [cax, fax, dax]]

    # Loop over upper mass limits, generating spectrum for each one
    for i, mc in enumerate(masslims):
        # set the upper stellar mass limit
        sps.params['masscut'] = mc
        # get the spectrum at age = tage and store it
        w, spec = sps.get_spectrum(tage=tage)
        mspec[:, i] = spec.copy()
        # plot the spectrum due to stars less massive than the limit
        cax.plot(w, spec, label='M < {}'.format(mc))

        if i == 0:
            # Plot the lowest mass bin spectrum
            dax.plot(w, spec, label='{} < M < {}'.format(0.08, mc))
            fax.plot(w, spec / spec_tot, label='{} < M < {}'.format(0.08, mc))
            # skip the rest of the loop
            continue

        # Subtract the spectrum from the last upper limit to ge tthe
        #  spectrum due to stars within the bin
        dmspec[:, i] = spec - mspec[:, i-1]
        # Plot it
        label = '{} < M < {}'.format(masslims[i-1], mc)
        dax.plot(w, dmspec[:, i], label=label)
        # Plot the total spectrum
        if mc == masslims[-1]:
            dax.plot(w, spec, label='Total', color='black')

        # plot the fractional
        fax.plot(w, dmspec[:, i] / spec_tot, label=label)

    # prettify the axes, titles, etc
    [ax.legend(loc=0) for ax in [cax, dax, fax]]
    [ax.set_xlabel('$\lambda (\AA)$') for ax in [cax, dax, fax]]
    [ax.set_ylabel('$F_\lambda$') for ax in [cax, dax]]
    [ax.set_yscale('log') for ax in [cax, dax]]
    [ax.set_ylim(1e-20, 1e-14) for ax in [cax, dax]]
    fstring = 'Age = {} Gyr, {}, log$Z/Z_\odot$={}'
    vals = 10.0, sfh[int(sps.params['const'])], sps.params['logzsol']
    [ax.set_title(fstring.format(*vals)) for ax in [cax, dax, fax]]
    fax.set_ylabel('$F/F_{tot}$')

    # plot filter transmission curves as shaded regions
    for f in filters:
        wavelength, transmission = f.transmission
        tmax = transmission.max()
        wmax = wavelength[transmission.argmax()]
        fax.fill_between(wavelength, transmission / tmax * 0.8,
                         alpha=0.3, color='grey')
        fax.text(wmax, 0.83, filterlist[f.name], fontdict={'size': 16})

    if savefig:
        # save to pdf
        cfig.savefig('cspec_'+name+'.pdf')
        dfig.savefig('dspec_'+name+'.pdf')
        ffig.savefig('fspec_'+name+'.pdf')

    return cfig, dfig, ffig


if __name__ == "__main__":

    # set the stellar population age
    tage = 10.0  # in Gyr

    # Parameters to loop over
    # ----------
    # log Z/Z_sun
    zs = [-1.0, 0.0]
    # Whether to include nebular emission
    nebs = [True, False]
    # Fraction of SF in constant SFR.  should be 0 or 1 for these plots
    const = [1.0]

    # loop over parameters, and show the fractional plot
    for z, n, c in product(zs, nebs, const):
        cf, df, ff = specplots(tage=tage, z=z, const=c, neb=n, savefig=False)
        ff.show()



# -*- coding: utf-8 -*-
"""
Tools for working with the FSPS filter set.

This module uses filter information shipped with FSPS itself in
``$SPS_HOME/data``.
"""

__all__ = ["find_filter", "FILTERS", "get_filter", "list_filters"]

import os
import numpy as np

# Cache for $SPS_HOME/data/magsun.dat parsed by numpy
MSUN_TABLE = None

# Cache for $SPS_HOME/data/filter_lambda_eff.dat parsed by numpy
LAMBDA_EFF_TABLE = None

# Cache for bandpass transmission listings: a dictionary keyed by bandpass
# name with values of wavelength, transmission tuples.
TRANS_CACHE = None


class Filter(object):
    def __init__(self, index, name, fullname):
        self.index = index - 1
        self.name = name.lower()
        self.fullname = fullname

    def __str__(self):
        return "<Filter({0})>".format(self.name)

    def __repr__(self):
        return "<Filter({0})>".format(self.name)

    @property
    def msun_ab(self):
        """Solar absolute magnitude in Filter, AB zeropoint."""
        # if self._msun_ab is None:
        if MSUN_TABLE is None:
            self._load_msun_table()
        if self.index < MSUN_TABLE.shape[0]:
            assert MSUN_TABLE[self.index, 0] == self.index + 1
            return float(MSUN_TABLE[self.index, 1])
        else:
            return np.nan

    @property
    def msun_vega(self):
        """Solar absolute magnitude in Filter, VEGAMAG zeropoint."""
        if MSUN_TABLE is None:
            self._load_msun_table()
        if self.index < MSUN_TABLE.shape[0]:
            assert MSUN_TABLE[self.index, 0] == self.index + 1
            return float(MSUN_TABLE[self.index, 2])
        else:
            return np.nan

    @property
    def lambda_eff(self):
        """Effective wavelength of Filter, in Angstroms."""
        if LAMBDA_EFF_TABLE is None:
            self._load_lambda_eff_table()
        if self.index < LAMBDA_EFF_TABLE.shape[0]:
            assert LAMBDA_EFF_TABLE[self.index, 0] == self.index + 1
            return float(LAMBDA_EFF_TABLE[self.index, 1])
        else:
            return np.nan

    @property
    def transmission(self):
        """Returns filter transmission: a tuple of wavelength (Angstroms) and
        an un-normalized transmission arrays.
        """
        if TRANS_CACHE is None:
            # Load the cache for all filters.
            self._load_transmission_cache()
        try:
            return TRANS_CACHE[self.name]
        except KeyError as e:
            e.args += "Could not find transmission data " "for {0}".format(self.name)
            raise

    def _load_msun_table(self):
        global MSUN_TABLE
        MSUN_TABLE = np.loadtxt(os.path.expandvars("$SPS_HOME/data/magsun.dat"))

    def _load_lambda_eff_table(self):
        global LAMBDA_EFF_TABLE
        LAMBDA_EFF_TABLE = np.loadtxt(
            os.path.expandvars("$SPS_HOME/data/filter_lambda_eff.dat")
        )

    def _load_transmission_cache(self):
        """Parse the allfilters.dat file into the TRANS_CACHE."""
        global TRANS_CACHE
        path = os.path.expandvars("$SPS_HOME/data/allfilters.dat")
        names = list_filters()
        TRANS_CACHE = {}
        filter_index = -1
        lambdas, trans = [], []
        with open(path) as f:
            for line in f:
                line.strip()
                if line[0].startswith("#"):
                    # Close out filter
                    if filter_index > -1:
                        TRANS_CACHE[names[filter_index]] = (
                            np.array(lambdas),
                            np.array(trans),
                        )
                    # Start new filter
                    filter_index += 1
                    lambdas, trans = [], []
                else:
                    try:
                        l, t = line.split()
                        lambdas.append(float(l))
                        trans.append(float(t))
                    except (ValueError):
                        pass
        # Close out the last filter
        TRANS_CACHE[names[filter_index]] = (np.array(lambdas), np.array(trans))


def _load_filter_dict():
    """
    Load the filter list, creating a dictionary of :class:`Filter` instances.
    """
    # Load filter table from FSPS
    filter_list_path = os.path.expandvars(
        os.path.join("$SPS_HOME", "data", "FILTER_LIST")
    )
    filters = {}
    with open(filter_list_path) as f:
        for line in f:
            columns = line.strip().split()
            if len(columns) > 1:
                fsps_id, key = columns[:2]
            else:
                continue
            comment = " ".join(columns[2:])
            filters[key.lower()] = Filter(int(fsps_id), key, comment)

    return filters


FILTERS = _load_filter_dict()


def find_filter(band):
    """
    Find the FSPS name for a filter.

    Usage:

    ::

        >>> import fsps
        >>> fsps.find_filter("F555W")
        ['wfpc2_f555w', 'wfc_acs_f555w']

    :param band:
        Something like the name of the band.

    """
    b = band.lower()
    possible = []
    for k in FILTERS.keys():
        if b in k:
            possible.append(k)
    return possible


def get_filter(name):
    """Returns the :class:`fsps.filters.Filter` instance associated with the
    filter name.

    :param name:
        Name of the filter, as found with :func:`find_filter`.
    """
    try:
        return FILTERS[name.lower()]
    except KeyError as e:
        e.args += (
            "Filter {0} does not exist. "
            "Try using fsps.find_filter('{0}').".format(name),
        )
        raise


def list_filters():
    """Returns a list of all FSPS filter names.

    Filters are sorted by their FSPS index.
    """
    lst = [(name, f.index) for name, f in FILTERS.items()]
    lst.sort(key=lambda x: x[1])
    return [l[0] for l in lst]



#!/usr/bin/env python

import os
import sys
import glob
import setuptools  # noqa: magic
import numpy.f2py
from numpy.distutils.core import setup, Extension
from numpy.distutils.command.build_ext import build_ext
from distutils.file_util import copy_file


if not os.path.exists("src/fsps/libfsps/src/sps_vars.f90"):
    raise RuntimeError(
        "It looks like FSPS is not included in your source distribution. "
        "If you're installing using git, don't forget to include the "
        "submodules using a recursive clone or by running: \n"
        "git submodule init\n"
        "git submodule update"
    )


numpy.f2py.run_main(
    [
        "-m",
        "_fsps",
        "-h",
        "src/fsps/_fsps.pyf",
        "--overwrite-signature",
        "src/fsps/fsps.f90",
    ]
)

ext = Extension(
    "fsps._fsps",
    sources=[
        "src/fsps/libfsps/src/sps_vars.f90",
        "src/fsps/libfsps/src/sps_utils.f90",
    ]
    + [
        f
        for f in glob.glob("src/fsps/libfsps/src/*.f90")
        if os.path.basename(f)
        not in [
            "autosps.f90",
            "simple.f90",
            "lesssimple.f90",
            "sps_vars.f90",
            "sps_utils.f90",
        ]
    ]
    + [
        "src/fsps/fsps.f90",
        "src/fsps/_fsps.pyf",
    ],
    extra_f90_compile_args=["-cpp"],
)


class custom_build_ext(build_ext):
    def run(self):
        build_ext.run(self)

        # HACKS: Copy over any extra DLL files
        # Note: numpy already tries to do this, but it doesn't do a very good job
        # when using the `src/pkgname` layout. We'll fix that here.
        pkg_roots = {
            os.path.abspath(os.path.dirname(self.get_ext_fullpath(ext.name)))
            for ext in self.extensions
        }
        for pkg_root in pkg_roots:
            shared_lib_dir = pkg_root
            for fn in os.listdir(self.extra_dll_dir):
                if not os.path.isdir(shared_lib_dir):
                    os.makedirs(shared_lib_dir)
                if not fn.lower().endswith(".dll"):
                    continue
                runtime_lib = os.path.join(self.extra_dll_dir, fn)
                copy_file(runtime_lib, shared_lib_dir)


# The final setup command. Note: we override the `build_ext` command with our
# custom version from above.
setup(
    name="fsps",
    url="https://github.com/dfm/python-fsps",
    author="Python-FSPS developers",
    author_email="foreman.mackey@gmail.com",
    description="Python bindings for Charlie Conroy's FSPS.",
    long_description=open("README.rst").read(),
    packages=["fsps"],
    package_dir={"": "src"},
    package_data={
        "": ["README.rst", "LICENSE.rst", "AUTHORS.rst"],
    },
    include_package_data=True,
    install_requires=["numpy"],
    ext_modules=[ext],
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Intended Audience :: Developers",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
    ],
    cmdclass={"build_ext": custom_build_ext},
    zip_safe=False,
)
